'''
This code is part of Cosmic Commando; Copyright (C) 2012 Piotr 'ZasVid' Sikora; see CosmicCommando.py for full notice.
'''

from guts.messages import message, a_or_an
from guts.game_data import game
from guts.utils import coinflip, random_pos, roll_dice, random_choice, eight_directions
from guts.ai import Confusion

def _damage(factor):
    return roll_dice(factor,7)

def _closest_agent(x, y, max_range):
    current_distance = 1
    while current_distance <= max_range:
        agents = [tile.occupant for tile in game.board.ring(x, y, current_distance) if tile.visible and tile.occupant is not None]
        if agents:
            return random_choice(agents)
        current_distance += 1
    return None

def throw_grenade(action, agent, target):
    weapon = action.piece
    hit = coinflip()
    x, y = target.x, target.y
    msg = ''
    if not hit:
        if agent.is_player():
            msg += "Your aim is a little off."
        deviation = random_choice(eight_directions)
        x += deviation[0]
        y += deviation[1]
    damage = _damage(action['damage'])
    msg += ("{0} throw{1} {2} {3}. It explodes, dealing {4} damage within blast radius ({5})!".format(
                agent.name.capitalize(), '' if agent.is_player() else 's',
                a_or_an(weapon.name), weapon.name, damage, action['radius']))
    message(msg)
    tiles_in_blast = game.board.circle(x,y,action['radius'])
    for tile in tiles_in_blast:
        if tile.occupant is not None:
            tile.occupant.take_damage(damage)

def mark_agent(action, agent, target):
    pass

def chain_lightning(action, agent):
    dmg_dice, current_range = action['damage'], action['range']
    x = agent.x
    y = agent.y
    while dmg_dice > 0:
        target = _closest_agent(x, y, current_range)
        if target is None:
            message('No enemy in range.', 'magenta')
            break
        else:
            dmg = _damage(dmg_dice)
            message('Electricity arcs to {0}. Damage due to electrocution is {1}.'.format(target.name, dmg), 'electric')
            target.take_damage(dmg)
            x, y = target.x, target.y
        dmg_dice -= 1

def shoot(action, agent, target):
    weapon = action.piece
    hit = coinflip()
    if hit:
        damage = _damage(action['damage'])
        message("{0} shoot{1} at {2}{3} with {4} {5} for {6} damage!".format(
                agent.name.capitalize(), '' if agent.is_player() else 's', target.name,
                'rself' if target.is_player() and agent.is_player() else '', a_or_an(weapon.name), weapon.name, damage))
        target.take_damage(damage)
    else:
        message("{0} miss{1} {2}.".format(agent.name.capitalize(), '' if agent.is_player() else 'es', target.name))

def confusing_melee(action, agent, target):
    target_health = target['health']
    melee(action, agent, target)
    if target_health > target['health'] and not target.has_flag('dead'):
        confusion_duration = (target_health - target['health']) / 4
        confusion_duration = confusion_duration if confusion_duration > 0 else 1 
        confusion_ai = Confusion(target['ai'], confusion_duration)
        target['ai'] = confusion_ai
        message('The eyes of the {0} look vacant, as he starts to stumble around! Confusion will last {1} turn{2}.'. \
                format(target.name, confusion_duration, 's' if confusion_duration > 1 else '')
                , 'light_green')    
    
def melee(action, agent, target):
    weapon = action.piece
    hit = coinflip() #TODO: replace with accuracy calculations
    if hit:
        damage = _damage(action['damage']) #TODO: replace with damage calculations
        message("{0} hit{1} {2} with {3} {4} for {5} damage!".format(
                agent.name.capitalize(), '' if agent.is_player() else 's', target.name, a_or_an(weapon.name), weapon.name, damage))
        target.take_damage(damage)
    else:
        message("{0} miss{1} {2}.".format(agent.name.capitalize(), '' if agent.is_player() else 'es', target.name))
        