'''
This code is part of Cosmic Commando; Copyright (C) 2012 Piotr 'ZasVid' Sikora; see CosmicCommando.py for full notice.
'''

from guts.debug import debug
from guts.game_data import game
from guts.messages import message, a_or_an
from guts.utils import coinflip, random_pos
from guts.game_piece import Piece
from guts.game_description import trait_level_description

item_action_types = ['ranged', 'melee', 'consumable', 'activated', 'deployable']

class Agent(Piece):
    
    def __init__(self, *args, **kwargs):
        Piece.__init__(self, *args, **kwargs)
        self['attitude'] = 'unknown'
        self.inventory = []
        for flag in item_action_types:
            self[flag + ' item'] = None

    def leave(self):
        game.board[self.x, self.y].occupant = None
        
    def _arrive_on(self, tile):
        tile.occupant = self
    
    def change_attitude(self, attitude):
        self['attitude'] = attitude

    def is_player(self):
        return self['attitude'] == 'player'

    def take_damage(self, damage):
        damaged_trait = None
        if damage > 0:
            if self.has('shields'):
                if damage > self['shields']:
                    damage -= self['shields']
                    self['shields'] = 0
                else:
                    self['shields'] -= damage
                    damage = 0
                    damaged_trait = 'shields'
            if self.has('armor'):
                damage = 0 if self['armor'] >= damage else damage - self['armor']
            if damage > 0:
                self['health'] -= damage
                if self['health'] <= 0:
                    self.death()
                    damaged_trait = None
                else:
                    damaged_trait = 'health'
        if damaged_trait is not None and not self.is_player():
            trd = trait_level_description(damaged_trait, self)
            message("{0} has {1}.".format(self.name.capitalize(), trd), trd)
        else:
            if not self.is_player() and not self.has_flag('dead'):
                message(self.name.capitalize() + " has too thick an armor to be hurt.", 'metallic gray')

    def death(self):
        message(self.name.capitalize() + " dies!", 'orange')
        #for item in [item for item in self.inventory]:
        #item.drop(self)
        self.become_corpse()
        self.set_flag('dead')
        self.ai = None
        self.leave()
    
    def become_corpse(self):
        #TODO: 
        '''monster.char = 'corpse'
        monster.color = 'dark_red'
        monster.name = 'remains of ' + monster.name
        '''
        pass

    def act(self):
        self['ai'].act(self)

    def wait(self):
        pass

    def drop(self, item):
        # remove from agent's inventory and put it on the tile the agent is standing on.
        self.inventory.remove(self)
        game.board[self.x, self.y].put(self)
        if self.is_player():
            message('You dropped a ' + self.name + '.')

    def pick_up(self, item):
        """ TODO: inventory limits or carrying capacity or both
        if agent.has_stat('carrying capacity'): 
        if len(Game().inventory) >= 26:
            message('Your inventory is full, cannot pick up ' + self.owner.name + '.', 'red')
        else:
        """
        game.board[item.x, item.y].remove(item)
        self.receive(item)
        if self.is_player:
            message('You picked up {0} {1}!'.format(a_or_an(item.name), item.name), 'green')

    def receive(self, item):
        #TODO: stackable items
        self.inventory.append(item)

    def equip(self, item):
        for action_type in item_action_types:
            if item.has(action_type):
                self[action_type + ' item'] = item
                #TODO: apply passive effects

    def melee(self, target = None):
        self['melee item'].use(self, 'melee', target)


class Player(Agent):

    def __init__(self, *args, **kwargs):
        Agent.__init__(self, *args, **kwargs)
        self['attitude'] = 'player'
        
    def death(self):
        message('You died!','blood red')
        game.state = 'dead'
        self.set_flag('dead')
        #for added effect, transform into a corpse!
        #TODO: actual dropping of a corpse in case of teamplay - probably just needs a switch in agent death()
        self.shape = 'corpse'
        self.color = 'blood red'
        message('Press <SPACE> or <ESC> to go back to main menu', 'important')

    def shoot(self, target):
        self['ranged item'].use(self, target)
        
