'''
This code is part of Cosmic Commando; Copyright (C) 2012 Piotr 'ZasVid' Sikora; see CosmicCommando.py for full notice.
'''

from guts.game_pieces import Terrain, Feature, Item
from guts.game_agents import Agent, Player, item_action_types
from guts.actions import new_action
from debug import debug
from ai import *

def new_player(name = "you"):
    player = Player(shape = 'player', color = 'dark red',
                    flags = ['blocking'],
                    traits = {'sight': 8, 'health' : 100, 'shields' : 100, 'melee damage': 18}
                    )
    for item_type in ['lightning launcher', 'psychodelic injector', 'blaster buzzard', 'grenade']:
        item = new_item(item_type)
        player.receive(item)
        player.equip(item)
    player['name'] = name
    return player

def _name_piece(piece, piece_type, name):
    if name is None:
        piece['name'] = piece_type
    else:
        piece['name'] = name


def new_item(item_type, name = None):
    """ Item factory.
    """
    shape, color, flags, traits = item_types.get(item_type,
                ('default', 'default', [], {}))
                #new_action('feedback', {'feedback msg': 'This doesn\'t do anything.'}
    if shape == 'default':
        debug.log("requested undefined item type: " + str(item_type))
    item = Item(shape = shape, color = color, flags = flags, traits = traits)
    for action_type in item_action_types:
        if item.has(action_type):
            action_key, action_data = traits[action_type]
            item[action_type] = new_action(action_key, action_data)
            item[action_type].piece = item
    _name_piece(item, item_type, name)
    return item
    
item_types = {'medkit' : ('medical', 'health', ['stacks'], 
                          {'consumable': ('replenish', {'trait': 'health', 'amount' : 50})}),
              'grenade' : ('grenade', 'military', ['stacks'], 
                           {'deployable': ('throw grenade', {'damage': 5, 'radius': 1})}),
              #ranged weapons
              'riven rifle' : ('ranged weapon', 'metallic', [], 
                               {'melee' : ('melee', {'damage' : 1}),
                                'ranged' : ('shoot', {'damage' : 3})}), 
              'blaster buzzard' : ('ranged weapon', 'metallic', [], { 'ranged': ('shoot', {'damage' : 4})}),
              'lightning launcher' : ('ranged weapon', 'electric', [],
                                    {#'melee': ('melee', {'damage' : 2}),
                                     #'ranged': ('mark agent', {}),
                                     'activated': ('chain lightning', {'damage' : 3, 'range': 5})}),
              #melee weapons
              'ravenous rapier': ('melee weapon', 'metallic', [], {'melee': ('melee', {'damage' : 2})}), #double vs health
              'gauntlet' : ('melee weapon', 'metallic', [], {'melee': ('melee', {'damage' : 2})}), 
              'power fist': ('melee weapon', 'metallic', [], {'melee': ('melee', {'damage' : 2})}), #double vs shields
              'psychodelic injector': ('melee weapon', 'metallic', [], {'melee': ('confusing melee', {'damage' : 2})}) #confuses on health hit
             }

def new_feature(feature_type, name = None):
    shape, color, flags, interaction = feature_types.get(feature_type,
                        ('default', 'default', [],
                          new_action('feedback', {'feedback msg': 'Default action.'})))
    if shape == 'default':
        debug.log("requested undefined feature type: " + str(feature_type))
    feature = Feature(shape = shape, color = color, flags = flags, interaction = new_action(interaction[0], interaction[1]))
    _name_piece(feature, feature_type, name)
    return feature

feature_types = {'closed door' : ('closed door', 'gray', ['closed', 'blocking', 'blocks sight'], ('door', {'open shape':'open door'}))
                }

def new_agent(agent_type, name = None):
    shape, color, flags, traits, equipment = agent_types.get((agent_type),
                        ('default', 'default', ['blocking'],
                         {'health': 100}, [])    
                        ) 
    if shape == 'default':
        debug.log("requested undefined agent type: " + str(agent_type))
    agent = Agent(shape = shape, color = color, flags = flags, traits = traits)
    for item_type in equipment:
        item = new_item(item_type)
        agent.receive(item)
        agent.equip(item)
    _name_piece(agent, agent_type, name)
    return agent

agent_types = {'Raven recruit': ('Raven soldier', 'Raven Rank 1', ['blocking'], {'health' : 50, 'shields': 25, 'ai': StandardSoldier()},
                                 ['riven rifle']),
               'Raven officer': ('Raven soldier', 'Raven Rank 2', ['blocking'], {'health' : 50, 'shields': 75, 'ai': StandardSoldier()},
                                 ['ravenous rapier', 'blaster buzzard']),
               'cybork' : ('cybork', 'metallic green', ['blocking'], {'health' : 100, 'armor':5, 'ai': RelentlessMelee()},
                                 ['power fist'])
              }
# give Raven officers Ravenous Rapiers as their melee attack

def new_terrain(terrain_type, name = None):
    shape, color, flags = terrain_types.get(terrain_type, ('default', 'default', []))
    if shape == 'default':
        debug.log("requested undefined terrain type: " + str(terrain_type))
    terrain = Terrain(shape, color, flags)
    _name_piece(terrain, terrain_type, name)
    return terrain

terrain_types = {'wall' : ('wall', 'metallic gray', ['blocking', 'blocks sight']),
                 'floor' : ('floor', 'metallic gray', []),
                }



    