# -*- coding: utf-8 -*-
'''
This code is part of Cosmic Commando; Copyright (C) 2012 Piotr 'ZasVid' Sikora; see CosmicCommando.py for full notice.
'''

import libtcodpy as libtcod
import os
from config import *
from ui import UI
from sys import exc_info
from gameplay import GamePlay
from achievements import Achievements

class GameSystem(object):
    '''
    This class handles setting the game up.
    '''

    def __init__(self, debug):
        self.debug = debug
        self.ui = UI()
        self.achievements = Achievements()

    def main_menu(self):
        img_left = libtcod.image_load(IMAGE_COSCOM)
        img_right = libtcod.image_load(IMAGE_RAVEN)
        while not libtcod.console_is_window_closed():
            #show the background image, at twice the regular console resolution
            self.ui.clear()
            #TODO: adjust for screen parameters
            libtcod.image_blit_2x(img_left, 0, 0, 2)
            libtcod.image_blit_2x(img_right, 0, 50, 2)
            
            #show the game's title, and some credits!
            libtcod.console_set_default_foreground(0, libtcod.light_yellow)
            libtcod.console_set_alignment(0, libtcod.CENTER)
            libtcod.console_print(0, SCREEN_WIDTH/2, SCREEN_HEIGHT/2 - 4,  'COSMIC COMMANDO')
            libtcod.console_print(0, SCREEN_WIDTH/2, SCREEN_HEIGHT-2, 'by ZasVid')
            #show options and wait for the player's choice
            #choice = self.ui.menu('', [('Instructions','i'), ('Play a new game', 'p'), ('Continue last game','c'), ('Quit','q')], 24, indexing = UI.MENU_CUSTOM_INDEX)
            choice = self.ui.menu('', ['Credits', 'Play a new game', 'Continue last game', 'Quit'], 24, indexing = UI.MENU_NUMERIC_INDEX)
            if choice == 0: #credits? manual? something like that
                self.ui.msgbox("Credits : \n" \
                               "ZasVid - game design, programming\n" \
                               #"Irinka - writing \n" \
                               "\nThis work contains bits of open-source code (BSD license) from: \n" \
                               "* Joao F. Henriques (a.k.a. Jotaf)'s python + libtcod tutorial \n" \
                               "\nThanks to: \n" \
                               "* contributors to http://roguebasin.roguelikedevelopment.org\n" \
                               "* adom.phx.pl for years of fanning the flame of roguelike passion\n" \
                               ,
                               width = SCREEN_WIDTH - 4,
                               text_color = 'light green' 
                               )#TODO: João
            elif choice == 1:  #new game
                self.gameplay = GamePlay(self.ui, self.achievements)
                self.gameplay.new()
                self.gameplay.play()
            elif choice == 2:  #load last game
                savefile = "savegame"
                if os.path.exists(savefile):
                    try:
                        self.gameplay = GamePlay(self.ui, self.achievements)
                        self.gameplay.load(savefile)
                        self.gameplay.play()
                    except:
                        self.ui.msgbox('\n No saved game to load.\n'+str(exc_info()), 24)
                        continue
                else:
                    self.ui.msgbox('\n No saved game to load.\n',)
            elif choice == 3:  #quit
                break
