'''
This code is part of Cosmic Commando; Copyright (C) 2012 Piotr 'ZasVid' Sikora; see CosmicCommando.py for full notice.
'''
from collections import defaultdict

        
def _keystr(keys):
    if None in keys: 
        return None
    return ' '.join(keys)

class InputInterpreter(object):
    """ Attention: characters that come from pressing shift + one of the special keys 
        (by libtcod's definition) need to be denoted as "shift [special key]". 
        E.g. @ doesn't work, "shift 2" does. Capital letters work properly.
    """
    def __init__(self):
        """
            #TODO: initialize inputs later from a file
        """
        self._inputs = {}
        self._direction_keys = defaultdict(list)
        self.hardcoded_inputs()
    
    def add_command(self, keystr, command):
        if command.startswith('DIRECTION '):
            self._direction_keys[command[10:]].append(keystr)
        self._inputs[keystr] = command

    def command(self, keys):
        return self._inputs.get(_keystr(keys), None) 

    def dirkeys(self, direction):
        return self._direction_keys[direction]

    def hardcoded_inputs(self):
        self.add_command('escape','QUIT')
        self.add_command('ctrl up','CAMERA NORTH')
        self.add_command('ctrl down','CAMERA SOUTH')
        self.add_command('ctrl left','CAMERA WEST')
        self.add_command('ctrl right','CAMERA EAST')
        self.add_command('left','DIRECTION WEST')
        self.add_command('right','DIRECTION EAST')
        self.add_command('up','DIRECTION NORTH')
        self.add_command('down','DIRECTION SOUTH')
        self.add_command('z','DIRECTION SOUTHWEST')
        self.add_command('x','DIRECTION SOUTH')
        self.add_command('c','DIRECTION SOUTHEAST')
        self.add_command('a','DIRECTION WEST')
        self.add_command('s','DIRECTION CENTER')
        self.add_command('d','DIRECTION EAST')
        self.add_command('q','DIRECTION NORTHWEST')
        self.add_command('w','DIRECTION NORTH')
        self.add_command('e','DIRECTION NORTHEAST')
        self.add_command('keypad 1','DIRECTION SOUTHWEST')
        self.add_command('keypad 2','DIRECTION SOUTH')
        self.add_command('keypad 3','DIRECTION SOUTHEAST')
        self.add_command('keypad 4','DIRECTION WEST')
        self.add_command('keypad 5','DIRECTION CENTER')
        self.add_command('keypad 6','DIRECTION EAST')
        self.add_command('keypad 7','DIRECTION NORTHWEST')
        self.add_command('keypad 8','DIRECTION NORTH')
        self.add_command('keypad 9','DIRECTION NORTHEAST')
        self.add_command('space','INTERACT')
        self.add_command('m','DISPLAY MESSAGES')
        self.add_command('shift 2','DISPLAY CHARACTER INFO')
        self.add_command('g','PICK UP')
        self.add_command('p','DROP')
        self.add_command('i','INVENTORY')
        self.add_command('r','USE ranged')
        self.add_command('v','USE consumable')
        self.add_command('f', 'USE deployable')
        self.add_command('tab', 'USE activated')
        self.add_command('left click', 'TARGET')
        self.add_command('right click', 'CANCEL')
        self.add_command('1', 'POWER 1')
        self.add_command('2', 'POWER 2')
        self.add_command('3', 'POWER 3')
        self.add_command('4', 'POWER 4')
        self.add_command('?', 'HELP')
