# coding=UTF-8
'''
This code is part of Cosmic Commando; Copyright (C) 2012 Piotr 'ZasVid' Sikora; see CosmicCommando.py for full notice.
'''

from config import *
from visuals import shape, color
from game_data import game
import libtcodpy as libtcod
from debug import debug
from ui_libtcod_keys import _special_keys

class UI(object):
    MENU_NUMERIC_INDEX = 'numeric'
    MENU_CUSTOM_INDEX = 'custom'
    MENU_ALPHABET_INDEX = 'alphabet'
    MAX_OPTIONS = {'numeric' : 10, 'alphabet' : 26, 'custom' : 255}
    
    def __init__(self):
        libtcod.console_set_custom_font(FONT, libtcod.FONT_TYPE_GRAYSCALE | libtcod.FONT_LAYOUT_TCOD)
        libtcod.console_init_root(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_TITLE, False)
        self.board_display = libtcod.console_new(CAMERA_WIDTH, CAMERA_HEIGHT)
        self.hud = libtcod.console_new(SCREEN_WIDTH, HUD_HEIGHT)
        libtcod.sys_set_fps(LIMIT_FPS)
        self.camera_x = 0
        self.camera_y = 0
        self.mouse_pos = None

    def clear(self):
        libtcod.console_set_default_background(0, color('black'))
        libtcod.console_clear(None)

    def camera_coordinates(self):
        return (self.camera_x, self.camera_y)

    def camera_coordinates_of(self, x, y):
        """
            TODO: check if necessary
            Convert coordinates on the map to coordinates on the screen.
        """
        (x, y) = (x - self.camera_x, y - self.camera_y)
        if (x < 0 or y < 0 or x >= CAMERA_WIDTH or y >= CAMERA_HEIGHT):
            return (None, None)  #if it's outside the view, return nothing
        return (x, y)

    def _set_camera_to(self, x, y):
        #make sure the camera doesn't see outside the map
        if x < 0: x = 0
        if y < 0: y = 0
        if x > game.board.width - CAMERA_WIDTH : x = game.board.width - CAMERA_WIDTH
        if y > game.board.height - CAMERA_HEIGHT: y = game.board.height - CAMERA_HEIGHT 
        (self.camera_x, self.camera_y) = (x, y)

    def center_camera(self, x, y):
        centered_x = x - CAMERA_WIDTH / 2  #coordinates so that the target is at the center of the screen
        centered_y = y - CAMERA_HEIGHT / 2
        self._set_camera_to(centered_x, centered_y)
    
    def move_camera(self, dx, dy):
        new_x = self.camera_x + dx
        new_y = self.camera_y + dy
        self._set_camera_to(new_x, new_y)

    def get_tile_under_mouse(self):
        mouse = libtcod.mouse_get_status()
        (x, y) = (mouse.cx, mouse.cy)
        (x, y) = (self.camera_x + x, self.camera_y + y)  #from screen to map coordinates
        return game.board[x,y]
    
    def render_all(self, center_on_player):
        if game.fov_recompute:
            #recompute FOV if needed (the player moved or something)
            libtcod.map_compute_fov(game.fov_map, game.player.x, game.player.y, game.player['sight'], FOV_LIGHT_WALLS, FOV_ALGO)
            libtcod.console_clear(self.board_display)
        if center_on_player:
            self.center_camera(game.player.x, game.player.y)
        self.render_board()
        self.render_hud()
        
    def _render_bar(self, x, y, total_width, name, value, maximum, bar_color, bkgnd_color):
        libtcod.console_set_alignment(self.hud, libtcod.CENTER)
        #render a bar (HP, experience, etc). first calculate the width of the bar
        bar_width = int(float(value) / maximum * total_width)
        #render the background first
        libtcod.console_set_default_background(self.hud, color(bkgnd_color))
        libtcod.console_rect(self.hud, x, y, total_width, 1, False, libtcod.BKGND_SET)
        #now render the bar on top
        libtcod.console_set_default_background(self.hud, color(bar_color))
        if bar_width > 0:
            libtcod.console_rect(self.hud, x, y, bar_width, 1, False, libtcod.BKGND_SET)
        #finally, some centered text with the values
        libtcod.console_set_default_foreground(self.hud, libtcod.white)
        
        libtcod.console_print(self.hud, x + total_width / 2, y,
                                     name + ': ' + str(value) + '/' + str(maximum))
    
    def clear_hud(self):
        #prepare to render the HUD
        libtcod.console_set_default_background(self.hud, libtcod.black)
        libtcod.console_clear(self.hud)
    
    def render_mouseover_info(self, only_this = True):
        #TODO: work it out
        #display names of objects under the mouse TODO: only once per render all? Not in a turn game, no.
        #libtcod.console_set_default_foreground(self.hud, color('light gray'))
        #libtcod.console_print(self.hud, 1, 0, str(self.get_tile_under_mouse()))
        #if only_this:
        #   libtcod.console_blit(self.hud, 0, 0, SCREEN_WIDTH, HUD_HEIGHT, 0, 0, HUD_Y)
        pass
    
    def render_hud(self):
        self.clear_hud()    
        #show the player's stats
        self._render_bar(1, 1, BAR_WIDTH, 'Health', game.player['health'], game.player['max health'], 'health', 'darker red')
        self._render_bar(1, 2, BAR_WIDTH, 'Shields', game.player['shields'], game.player['max shields'], 'shields', 'darker blue')
#        libtcod.console_print_left(self.panel, 1, 3, libtcod.BKGND_NONE, 'Dungeon level ' + str(Game.level))
        #print the game messages, one line at a time
        libtcod.console_set_alignment(self.hud, libtcod.LEFT)
        y = 1
        for (line, msg_color) in game.msgs:
            libtcod.console_set_default_foreground(self.hud, color(msg_color))    
            libtcod.console_print(self.hud, MSG_X, y, line)
            y += 1
        #blit the contents of HUD to the root console
        libtcod.console_blit(self.hud, 0, 0, SCREEN_WIDTH, HUD_HEIGHT, 0, 0, HUD_Y)

    def render_board(self):
        for y in range(min(CAMERA_HEIGHT, game.board.height)):
            for x in range(min(CAMERA_WIDTH, game.board.width)):
                (board_x, board_y) = (self.camera_x + x, self.camera_y + y)
                visible = libtcod.map_is_in_fov(game.fov_map, board_x, board_y)
                tile = game.board[board_x, board_y]
                tile.visible = visible
                libtcod.console_put_char_ex(self.board_display, x, y, shape(tile.shape), color(tile.color), color(tile.highlight))
        libtcod.console_blit(self.board_display, 0, 0, CAMERA_WIDTH, CAMERA_HEIGHT, 0, 0, 0)

    def menu(self, header, options, width = MSG_BOX_WIDTH, indexing = MENU_ALPHABET_INDEX, text_color = 'white'):
        """ TODO: make a listbox with elements that are not selectable to display e.g. message archive
            TODO: extend the menu to be scrollable (and be capable of containing more
                 options than the height of the screen would allow - 
                 renumeration of options to those fitting the screen a la ADOM would work.
            TODO: support colorful_options (the options would then come in tuples of size 2: text + color,
                  or 3 in case of custom indexing (option, text, color)
                 text_color would be used for header and if an options' color is None)
            TODO: allow multiple keys for an option in custom indexing
        """
        
        max_options = UI.MAX_OPTIONS[indexing]
        if len(options) > max_options:
            raise ValueError('Cannot have a menu with ' + indexing + \
                             " indexing with more than " + str (max_options) + ' options.')
        #calculate total height for the header (after auto-wrap) and one line per option
        #TODO: does it wrap?
        header_height = libtcod.console_get_height_rect(self.board_display, 0, 0, width, SCREEN_HEIGHT, header)
        if header == '':
            header_height = 0
        height = len(options) + header_height
        #create an off-screen console that represents the menu's window
        window = libtcod.console_new(width, height)
        #print the header, with auto-wrap
        libtcod.console_set_default_foreground(window, color(text_color))
        libtcod.console_set_alignment(window, libtcod.LEFT)
        libtcod.console_print_rect(window, 0, 0, width, height, header)
        #print all the options
        y = header_height
        if indexing == UI.MENU_ALPHABET_INDEX:
            start_index = ord('a')
        else:
            start_index = ord('0')
        option_indices = []
        if indexing == UI.MENU_CUSTOM_INDEX:
            for option_text, option_index in options:
                text = '(' + option_index + ') ' + option_text
                if ord(option_index) in option_indices:
                    debug.log("Menu: " + header + " contains two options with the same index: " + option_index)
                option_indices.append(ord(option_index))
                libtcod.console_print_left(window, 0, y, libtcod.BKGND_NONE, text)
                y += 1
        else:
            letter_index = start_index
            for option_text in options:
                text = '(' + chr(letter_index) + ') ' + option_text
                libtcod.console_print(window, 0, y, text)
                y += 1
                letter_index += 1
        #blit the contents of "window" to the root console
        x = SCREEN_WIDTH/2 - width/2
        y = SCREEN_HEIGHT/2 - height/2
        libtcod.console_blit(window, 0, 0, width, height, 0, x, y, 1.0, 0.7)
        #present the root console to the player and wait for a key-press
        libtcod.console_flush()
        key, mouse = libtcod.Key(), libtcod.Mouse()
#        while not libtcod.console_is_window_closed():
        libtcod.sys_wait_for_event(libtcod.EVENT_KEY_RELEASE | libtcod.EVENT_MOUSE_RELEASE, key, mouse, True)
        #convert the ASCII code to an index; if it corresponds to an option, return it
        if indexing == UI.MENU_CUSTOM_INDEX:
            for i in range(len(option_indices)):
                if key.c == option_indices[i]:
                    return i
        else:
            index = key.c - start_index
            if index >= 0 and index < len(options): return index
        return None

    def listbox(self, text, width=MSG_BOX_WIDTH, text_color = 'white'):
        pass

    def msgbox(self, text, width=MSG_BOX_WIDTH, text_color = 'white'):
        self.menu(text, [], width, text_color = text_color)  #use menu() as a sort of "message box" 

    def pointer_selection(self):
        """ Returns an element of the interface pointed at by the mouse cursor.
            This might me a tile, a menu line or something else.
            TODO: implement
        """
        pass

    def get_input(self):
        """ Alternatively, CTRL and ALT could be left out of this, disallowed as modifiers
            and used for commands (crouch!). Holding the keys for an auxiliary command would
            also be possible and perhaps even useful in a real-time roguelike.
        """
        """ Mouse status from libtcod contains fields:
            x,y,dx,dy - position and change of position in pixels
            cx, cy, dcx, dcy - position and change of position in cells - note: dcx, dcy might not update properly according to a forum thread
            lbutton, rbutton, mbutton - True if the button is still pressed
            lbutton_pressed, rbutton_pressed, mbutton_pressed - True if there was a click (press and release)
            wheel_up, wheel_down - undocumented, looks to be boolean, so I guess this would be "has the wheel been scrolled up/down since last check?"
        """

        key, mouse, inputs = libtcod.Key(), libtcod.Mouse(), []
        libtcod.sys_check_for_event(libtcod.EVENT_KEY_RELEASE | libtcod.EVENT_MOUSE, key, mouse)
        self.mouse_pos = (mouse.cx, mouse.cy)
        if key.lalt or key.ralt:
            inputs.append('alt')
        if key.rctrl or key.lctrl:
            inputs.append('ctrl')
        if key.vk == libtcod.KEY_CHAR:
            inputs.append(chr(key.c))
        elif key.vk not in [libtcod.KEY_NONE, libtcod.KEY_CONTROL, libtcod.KEY_ALT, libtcod.KEY_SHIFT]:
            if key.shift:
                inputs.append('shift')
            inputs.append(_special_keys[key.vk])
        else:
            if mouse.lbutton_pressed:
                inputs.append('left click')
            elif mouse.rbutton_pressed:
                inputs.append('right click')
            elif mouse.mbutton_pressed:
                inputs.append('middle click')
        #print(str(inputs))
        return inputs
     
